# state51-logger

This project provides the state51-logger, a set of perl classes giving you :-

* a generalised logging library
* a connector to various logging backends, notably RabbitMQ but also the Console
* an event hierarchy
* a way of requesting Jobs in the state51 platform Job Execution system

There are four levels of message, "warn", "fatal", "info" and "trace".

If you're using the Console Backend :-

* "info" will also be printed on STDOUT, "warn" and "fatal" will go to STDERR.
* "trace" will go to STDERR, but only if the log object has been initialised with a true value for trace.
* "fatal" message will cause a fatal croak (after the message + croak message have been logged)

## Log Something

```perl
#!/usr/bin/env perl

use strict;
use warnings;
use feature ':5.10';

use state51::Logger;

my $logger = state51::Logger->new();

$logger->message(
                 level   => "info",
                 message => ["Test Message From Andy"]
                );
```

```perl
#!/usr/bin/env perl

use strict;
use warnings;
use feature ':5.10';

use state51::Logger;

$opts{HandleError} = sub {
    state51::Logger->new->message( level => 'warn', message => [ 'DB error', $_[0] ] );
    return;
};
```

## Create an Event

Events are sent to indicate something has happened - a script has started running, a problem has occured, a file has been improted for an organisation. Events are classes defined in the `state51::Logger` project. Events are serialised to JSON to be delivered to RabbitMQ using MooseX::Storage.

```perl
#!/usr/bin/env perl
use strict;
use warnings;
use feature ':5.10';

use state51::Logger;
use state51::Logger::Event::Generic;

my $logger = state51::Logger->new();

$logger->message(
            level   => "info",
            message => [
                state51::Logger::Event::Generic->new(
                    message => "My Generic Test Message",
                )
            ]
);
```

```perl
#!/usr/bin/env perl
use strict;
use warnings;
use feature ':5.10';

use state51::Logger;
use state51::Logger::Event::ScriptStart;

my $logger = state51::Logger->new();

$logger->message(
            level   => "info",
            message => [
                state51::Logger::Event::ScriptStart->new(
                    name => 'demoscript.pl',
                    user => 'andym',
                    OriginatingOrganisationID => 8632477670016,
                    ApplicationID => 54838309120320,
                    Environment => 'production',
                )
            ]
);
```

## Create a Job

Sending a Job requests that some work should happen. This could be a request for a format to have QA run, a file to be encoded, a mailout to be sent. This work will happen asynchronously on the platform Job runner system. 
This script does a `use state51::Base`, and imports the whole state51 platform. That system defines the Jobs and also has the Job runners. Your Job is serialised into a message and sent to MQ if you have your connector configured. The other logging examples send messages to the MQ `firehose` exchange, Jobs go to Job specific exchanges, defined by the job class.

```perl
#!/usr/bin/perl
use strict;
use warnings;
use feature ':5.10';

use state51::Base;
state51::Base->connect();

use state51::Job::RunQAForFormat;

my @format_ids = (63051781410032, 63051783610032);

my $job = state51::Job::RunQAForFormat->new(format_oid => \@format_ids);
$job->log_message(level => 'info', message => [$job]);
```

## Config

Configure your logging Backend

```
export PLATFORM_LOG_BACKENDS=Console,RabbitMQ
```

Set up your rabbit mq connection

```
export PLATFORM_MQ_SERVER=mq.cissme.com
export PLATFORM_MQ_PORT=5672
export PLATFORM_MQ_USER=platform
export PLATFORM_MQ_PASSWORD=reallygoodpw
```
