#!/usr/bin/env perl

use strict;
use warnings;
use feature ":5.10";

use IO::File;
use RDF::Helper;
use RDF::Helper::Constants qw(:rdf :rdfs);

use Moose::Util qw/find_meta does_role search_class_by_role/;
use Module::Find;

my $omd_location = "https://openmusicdata.org/ns/omde/";
my $output_location = '../../graphql-schemas/sources/omd_event/';

my @event_classes = useall state51::Logger::Event;

foreach my $class (sort @event_classes) {
	generate_ttl_for_event_class($class);
}

sub generate_ttl_for_event_class {
    my ($class) = @_;

    my $meta = find_meta($class);

    # you can't make roles directly, so they're not useful to diagram?
    return '' if is_role($class);

    my $short_classname = $class;
    $short_classname =~ s/state51::Logger::Event:://;
    $short_classname =~ s/:://;

    my $rdf = RDF::Helper->new(
        BaseInterface => 'RDF::Trine',
        namespaces    => {
            'omd' => $omd_location,
            rdf   => RDF_NS,
            rdfs  => RDFS_NS,
            'xsd' => 'http://www.w3.org/2001/XMLSchema#',
            'schema' => 'http://schema.org/',
        },
        ExpandQNames => 1,
        base_uri     => $omd_location,
    );

    $rdf->assert_resource('omd:' . $short_classname, 'rdf:type', RDFS_CLASS);
    $rdf->assert_literal('omd:' . $short_classname, RDFS_LABEL, $short_classname);
    # assume that a documenation sub returns a string
    if (($class->can('documentation') and ($class->documentation()))) {
        $rdf->assert_literal('omd:' . $short_classname,
            RDFS_COMMENT, $class->documentation());
    }

    # FIXME - are we a subclass?
    foreach my $attribute ($meta->get_all_attributes) {
        my $attribute_name = $attribute->name();

        my $subject = 'omd:' . camelize($attribute_name);

        # 'rdf:type'='a'
        $rdf->assert_resource($subject, 'rdf:type', RDF_PROPERTY);
        $rdf->assert_literal($subject, RDFS_LABEL, $attribute_name);
        $rdf->assert_resource($subject, 'schema:domainIncludes',
            'omd:' . $short_classname);
        if ($attribute->has_type_constraint) {
            my @type = @{generate_type($attribute->type_constraint, $attribute_name)};
            if (@type) {
                foreach my $type (@type) {
                    $rdf->assert_resource($subject, 'schema:rangeIncludes',
                        $type);
                }
            }
        }
        if ($attribute->documentation) {
            $rdf->assert_literal($subject, RDFS_COMMENT, $attribute->documentation);
        }
    }

    my $fh = IO::File->new(
        $output_location . camelize($short_classname) . '.ttl', "w");
    $rdf->serialize(filename => $fh, format => 'turtle');
}

sub is_role {
    my $package_or_obj = shift;

    my $meta = find_meta($package_or_obj);
    return if not $meta;
    return $meta->isa('Moose::Meta::Role');
}

sub camelize {
    my $s = shift;
    $s = join('', map { ucfirst $_ } split(/(?<=[A-Za-z])_(?=[A-Za-z])|\b/, $s));
    $s =~ s/\W//g; # remove non-word characters

    return $s;
}

# turn moose type_constaint into a type
# see https://www.w3.org/TR/xmlschema-2/#built-in-datatypes
sub generate_type {
    my ($type_constraint, $attribute_name) = @_;

    my @type;
    foreach my $moose_type (split /\|/, $type_constraint) {
        if (($moose_type eq 'Str') or ($moose_type eq 'Maybe[Str]')) {
            @type = ('xsd:string');
        }
        elsif ($moose_type eq 'state51::OrganisationAttribute::Retailer') {
            @type = ('omd:Retailer');
        }
        elsif (($moose_type eq 'Int')
            or ($moose_type eq 'Num')
            or ($moose_type eq 'Maybe[Int]')
            or ($moose_type eq 'ArrayRef[Int]'))
        {
            @type = ('xsd:positiveInteger');
        }
        elsif ($moose_type eq 'MooseX::Types::ISO8601::ISO8601DateTimeStr') {
            @type = ('xsd:dateTime');
        }
        elsif (($moose_type eq 'Undef')
            or ($moose_type eq 'HashRef')
            or ($moose_type eq '__ANON__')
            or ($moose_type eq 'Object'))
        {
            # um
        }
        else {
            warn "unmapped type: " . $moose_type . " for " . $attribute_name;
        }
    }

    return \@type;
}
