package state51::Logger::RabbitMQ;
use Moose;
use MooseX::Types::Moose qw/Str Int/;
use namespace::autoclean;
use Net::RabbitMQ;
use Data::UUID;

with 'state51::Logger::Role';

has correlation_id => (
    default => sub { my $ug = Data::UUID->new; my $uuid = $ug->create; $ug->to_string($uuid); },
    isa     => Str,
    is      => 'rw',
);

has mq_host => (
    isa     => Str,
    default => sub { $ENV{PLATFORM_MQ_SERVER} },
    lazy    => 1,
    is      => 'ro'
);

has mq_port => (
    isa     => Int,
    default => sub { $ENV{PLATFORM_MQ_PORT} || 5672 },
    is      => 'ro'
);

has mq_user => (
    isa     => Str,
    default => sub { $ENV{PLATFORM_MQ_USER} },
    lazy    => 1,
    is      => 'ro',
);

has mq_password => (
    isa     => Str,
    default => sub { $ENV{PLATFORM_MQ_PASSWORD} },
    lazy    => 1,
    is      => 'ro',
);

has mq => (
    isa       => 'Net::RabbitMQ',
    builder   => '_build_mq',
    lazy      => 1,
    is        => 'ro',
    predicate => 'has_mq',
);

sub _build_mq {
    my ($self) = @_;
    my $mq = Net::RabbitMQ->new();
    Carp::confess("set mq user/password")
      unless $self->mq_user && $self->mq_password;
    $self->connect($mq);
    return $mq;
}

sub connect {
    my ( $self, $mq ) = @_;

    $mq->connect(
        $self->mq_host => {
            user        => $self->mq_user,
            password    => $self->mq_password,
            channel_max => 2,
            timeout     => 5,
        }
    );
    $mq->channel_open(1);
}

sub DESTROY {
    my ($self) = @_;
    return unless $self->has_mq;
    my $mq = $self->mq();
    $mq->disconnect();
}

sub publish_message {
    my $self = shift;
    my (%p) = @_;
    $p{body} = blessed( $p{body} ) ? $p{body}->freeze : $p{body};

    my $options = { exchange     => $p{exchange} };
    my $props   = { content_type => 'application/json', correlation_id => $self->correlation_id() };
    if ($p{reply_to}) {
      $props->{'reply_to'} = $p{reply_to};
    }

    my $resp =
      $self->mq->publish( 1, $p{routing_key}, $p{body}, $options, $props );
    if ( $resp != 0 ) {
        print("TRYING RECONNECT\n");
        my $mq = $self->mq;
        $self->connect($mq);
        $self->mq->publish( 1, $p{routing_key}, $p{body}, $options, $props );
    }
}

__PACKAGE__->meta->make_immutable( inline_destructor => 0 );
1;
